﻿//---------------------------------------------------------------------------
//
//   ImportDXF
//
//          Copyright(C) 1999-2016, tetraface Inc.
//
//      Sample of Import plug-in for importing a DXF (AutoCAD) file.
//      Created DLL must be installed in "Plugins\Import" directory.
//      This is just a sample. Metasequoia already contains the embedded 
//     DXF importer in the main body. There is no means to use this plug-in.
//
//    　DXF(AutoCAD)形式を入力するインポートプラグインのサンプル。
//    　作成したDLLは"Plugins\Import"フォルダに入れる必要がある。
//    　DXF形式自体がＣＡＤ向けの機能しか提供していないのと、加えて仕様の詳細
//    を知らないため、このサンプルではあまり多くの情報を入力していない。
//    なお、これは単なるサンプルであって、Metasequoiaの内部にはDXFインポート
//    機能が組み込まれているので、このプロジェクトで作成したプラグインを使う
//    意味はない。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "MQPlugin.h"
#include "Strings.h"

BOOL LoadDXF(const char *filename, MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x4E0A0173;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "AutoCAD DXF Importer   Copyright(C) 1999-2016, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// 入力用プラグインである
	return MQPLUGIN_TYPE_IMPORT;
}

//---------------------------------------------------------------------------
//  MQEnumFileType
//    入力または出力可能なファイルタイプを返す。
//    ファイルタイプは別名保存時のダイアログに表示される。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumFileType(int index)
{
	switch(index){
	case 0: return "AutoCAD DXF (*.dxf)";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQEnumFileExt
//    入力または出力可能な拡張子を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumFileExt(int index)
{
	switch(index){
	case 0: return "dxf";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQImportFile
//    [読み込み]やドラッグ＆ドロップで読み込むときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQImportFile(int index, const char *filename, MQDocument doc)
{
	switch(index){
	case 0: return LoadDXF(filename, doc);
	}
	return FALSE;
}


// DXF形式を入力
BOOL LoadDXF(const char *filename, MQDocument doc)
{
	int index[4], vert_count;
	int sc;
	MQPoint p;

	//DXFファイルをオープン
	FILE *fh;
	errno_t ret_s = fopen_s(&fh, filename, "rb");
	if(ret_s != 0)
		return FALSE;

	// Strings.cppで定義されているテキストファイルの読み込み用クラスを作成
	FileStringSeparator *afs = new FileStringSeparator(fh);

	// オブジェクトの作成
	MQObject obj = MQ_CreateObject();
	//MQObj_SetName(obj, ""); //このサンプルでは名前は付けないのでコメントアウト

	for(int count=0; ; count++)
	{
		sc = afs->GetLine();
		if(sc == EOF)break;
		if(sc == 0)continue;

		if(afs->chkcom("3DFACE",1))
		{
			for(vert_count=0;;)
			{
				sc = afs->GetLine();
				if(sc == EOF)break;
				if(sc == 0)continue;

				int comid = afs->ToInt(0);
				switch(comid){
				case 0: //end
					// ３点以上頂点があれば、面を追加
					if(vert_count >= 3)
						obj->AddFace(vert_count, index);
					goto _3DFACE_END;
				case 10:
				case 11:
				case 12:
				case 13:
					sc = afs->GetLine();
					if(sc == EOF) goto _3DFACE_END;
					if(sc > 0)
						p.x = afs->ToFloat(0);
					break;
				case 20:
				case 21:
				case 22:
				case 23:
					sc = afs->GetLine();
					if(sc == EOF) goto _3DFACE_END;
					if(sc > 0)
						p.y = afs->ToFloat(0);
					break;
				case 30:
				case 31:
				case 32:
				case 33:
					sc = afs->GetLine();
					if(sc == EOF) goto _3DFACE_END;
					if(sc > 0)
						p.z = afs->ToFloat(0);
					// 頂点がまだ４個登録されていなければ、頂点を追加
					if(vert_count < 4)
						index[vert_count++] = obj->AddVertex(p);
					break;
				//case 8: //layer command
				//case 62: //color command
				default:
					// 不明なコマンドを無視する
					if(afs->GetLine() == EOF) goto _3DFACE_END;
				}
			}
_3DFACE_END:;
		}
	}

	delete afs;

	fclose(fh);

	// もし面がまったく登録されていないならFALSEを返す
	if(obj->GetFaceCount() == 0)
	{
		obj->DeleteThis();
		return FALSE;
	}

	// 各ポリゴンはバラバラであるため、同じ位置(実際には小数点誤差を考慮して
	// 0.0001以下の距離)にあるすべて頂点を結合する
	obj->OptimizeVertex(0.0001f, NULL);

	// オブジェクトをドキュメントへ登録
	doc->AddObject(obj);

	return TRUE;
}

