﻿//---------------------------------------------------------------------------
//
//   Spring
//
//          Copyright(C) 1999-2016, tetraface Inc.
//
//     Sample of Create plug-in for creating a spring.
//     Created DLL must be installed in "Plugins\Create" directory.
//
//    　バネを作成するクリエイトプラグインのサンプル。
//    　作成したDLLは"Plugins\Create"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "MQPlugin.h"
#include "MQWidget.h"
#include "resource.h"

HINSTANCE hInstance;

BOOL CreateSpring(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//ダイアログボックスの表示に必要なので、インスタンスを保存しておく
	hInstance = (HINSTANCE)hModule;

	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x70AE0C31;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Create spring           Copyright(C) 1999-2016, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// 作成用プラグインである
	return MQPLUGIN_TYPE_CREATE;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Create spring";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQCreate
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQCreate(int index, MQDocument doc)
{
	switch(index){
	case 0: return CreateSpring(doc);
	}
	return FALSE;
}

class SpringDialog : public MQDialog
{
public:
	SpringDialog(MQWindowBase& parent);
	~SpringDialog();

	MQDoubleSpinBox *edit_length;
	MQDoubleSpinBox *edit_rotate;
	MQDoubleSpinBox *edit_radius;
	MQSpinBox *edit_seg_u;
	MQSpinBox *edit_seg_v;
	MQDoubleSpinBox *edit_width;
};

SpringDialog::SpringDialog(MQWindowBase& parent) : MQDialog(parent)
{
	wchar_t buffer[256];

	LoadStringW(hInstance, IDS_TITLE, buffer, _countof(buffer));
	SetTitle(buffer);

	MQFrame *mainFrame = CreateHorizontalFrame(this);

	MQFrame *paramFrame = CreateHorizontalFrame(mainFrame);
	paramFrame->SetMatrixColumn(2);
	
	LoadStringW(hInstance, IDS_LENGTH, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_length = CreateDoubleSpinBox(paramFrame);
	edit_length->SetMin(0);
	edit_length->SetMax(1e+8);
	edit_length->SetPosition(100);

	LoadStringW(hInstance, IDS_ROTATE, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_rotate = CreateDoubleSpinBox(paramFrame);
	edit_rotate->SetMin(0);
	edit_rotate->SetMax(360);
	edit_rotate->SetPosition(8);

	LoadStringW(hInstance, IDS_RADIUS, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_radius = CreateDoubleSpinBox(paramFrame);
	edit_radius->SetMin(0);
	edit_radius->SetMax(1e+8);
	edit_radius->SetPosition(20);

	LoadStringW(hInstance, IDS_SEG_U, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_seg_u = CreateSpinBox(paramFrame);
	edit_seg_u->SetMin(1);
	edit_seg_u->SetMax(10000);
	edit_seg_u->SetPosition(100);

	LoadStringW(hInstance, IDS_SEG_V, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_seg_v = CreateSpinBox(paramFrame);
	edit_seg_v->SetMin(3);
	edit_seg_v->SetMax(1000);
	edit_seg_v->SetPosition(100);

	LoadStringW(hInstance, IDS_WIDTH, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_width = CreateDoubleSpinBox(paramFrame);
	edit_width->SetMin(0);
	edit_width->SetMax(1e+8);
	edit_width->SetPosition(8);

	MQFrame *sideFrame = CreateVerticalFrame(mainFrame);

	LoadStringW(hInstance, IDS_OK, buffer, _countof(buffer));
	MQButton *okbtn = CreateButton(sideFrame, buffer);
	okbtn->SetDefault(true);
	okbtn->SetModalResult(MQDialog::DIALOG_OK);

	LoadStringW(hInstance, IDS_CANCEL, buffer, _countof(buffer));
	MQButton *cancelbtn = CreateButton(sideFrame, buffer);
	cancelbtn->SetCancel(true);
	cancelbtn->SetModalResult(MQDialog::DIALOG_CANCEL);
}

SpringDialog::~SpringDialog()
{
}


struct spring_param {
	float length;
	float rotate;
	float radius;
	int seg_u;
	int seg_v;
	float width;
};

// バネを作成
BOOL CreateSpring(MQDocument doc)
{
	int u,v;
	int mat_index;

	// ダイアログボックスを表示
	MQWindow mainwin = MQWindow::GetMainWindow();
	SpringDialog dlg(mainwin);
	if(dlg.Execute() != MQDialog::DIALOG_OK){
		return FALSE;
	}

	spring_param spring;
	spring.length = (float)dlg.edit_length->GetPosition();
	spring.rotate = (float)dlg.edit_rotate->GetPosition();
	spring.radius = (float)dlg.edit_radius->GetPosition();
	spring.seg_u = dlg.edit_seg_u->GetPosition();
	spring.seg_v = dlg.edit_seg_v->GetPosition();
	spring.width = (float)dlg.edit_width->GetPosition();

	// バネ用のオブジェクトを作成
	MQObject obj = MQ_CreateObject();

	// 頂点を作成
	for(u=0; u<=spring.seg_u; u++)
	{
		for(v=0; v<spring.seg_v; v++)
		{
			MQPoint p;
			float u_angle = (float)u / spring.seg_u * spring.rotate * 2 * 3.14159f;
			float v_angle = (float)v / spring.seg_v * 2 * 3.14159f;

			p.y = (float)spring.length * u / spring.seg_u + (float)cos(v_angle) * spring.width;
			p.x = (float)cos(u_angle) * (spring.radius + (float)sin(v_angle) * spring.width);
			p.z = (float)sin(u_angle) * (spring.radius + (float)sin(v_angle) * spring.width);
			obj->AddVertex(p);
		}
	}

	// カレントマテリアルの番号を得る
	mat_index = doc->GetCurrentMaterialIndex();

	// 面を作成
	for(u=0; u<spring.seg_u; u++)
	{
		for(v=0; v<spring.seg_v; v++)
		{
			int index[4];
			index[0] = (u  )*spring.seg_v + v;
			index[1] = (u  )*spring.seg_v + (v+1)%spring.seg_v;
			index[2] = (u+1)*spring.seg_v + (v+1)%spring.seg_v;
			index[3] = (u+1)*spring.seg_v + v;
			// 面を追加
			int face_index = obj->AddFace(4, index);
			// 追加した面にカレントマテリアルを割り当て
			obj->SetFaceMaterial(face_index, mat_index);
		}
	}

	// オブジェクトをドキュメントへ登録
	doc->AddObject(obj);

	return TRUE;
}

 
